package cn.ibizlab.core.ou.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.HashSet;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.ou.domain.IBZDepartment;
import cn.ibizlab.core.ou.filter.IBZDepartmentSearchContext;
import cn.ibizlab.core.ou.service.IIBZDepartmentService;

import cn.ibizlab.util.helper.CachedBeanCopier;



import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.ou.mapper.IBZDepartmentMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;


/**
 * 实体[部门] 服务对象接口实现
 */
@Slf4j
@Service("IBZDepartmentServiceImpl")
public class IBZDepartmentServiceImpl extends ServiceImpl<IBZDepartmentMapper, IBZDepartment> implements IIBZDepartmentService {

    @Autowired
    @Lazy
    private cn.ibizlab.core.ou.service.IIBZDepartmentService ibzdepartmentService;
    @Autowired
    @Lazy
    private cn.ibizlab.core.ou.service.IIBZEmployeeService ibzemployeeService;
    private int batchSize = 500;

    @Override
    @Transactional
    public boolean create(IBZDepartment et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getDeptid()),et);
        return true;
    }

    @Override
    public void createBatch(List<IBZDepartment> list) {
        saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public IBZDepartment get(String key) {
        IBZDepartment et = getById(key);
        if(et==null){
            et=new IBZDepartment();
            et.setDeptid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean save(IBZDepartment et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(IBZDepartment et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<IBZDepartment> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    public boolean checkKey(IBZDepartment et) {
        return (!ObjectUtils.isEmpty(et.getDeptid()))&&(!Objects.isNull(this.getById(et.getDeptid())));
    }

    @Override
    public IBZDepartment getDraft(IBZDepartment et) {
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        if(!removeById(key)) return false;
        return true ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean update(IBZDepartment et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("deptid",et.getDeptid())))
            return false;
        CachedBeanCopier.copy(get(et.getDeptid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<IBZDepartment> list) {
        updateBatchById(list,batchSize);
    }


	@Override
    public List<IBZDepartment> selectByParentdeptid(String deptid) {
        return baseMapper.selectByParentdeptid(deptid);
    }

    @Override
    public void removeByParentdeptid(String deptid) {
        this.remove(new QueryWrapper<IBZDepartment>().eq("pdeptid",deptid));
    }

	@Override
    public List<IBZDepartment> selectByOrgid(String orgid) {
        return baseMapper.selectByOrgid(orgid);
    }

    @Override
    public void removeByOrgid(String orgid) {
        this.remove(new QueryWrapper<IBZDepartment>().eq("orgid",orgid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<IBZDepartment> searchDefault(IBZDepartmentSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZDepartment> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZDepartment>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }


    @Override
    public List<JSONObject> select(String sql){
        return this.baseMapper.selectBySQL(sql);
    }

    @Override
    @Transactional
    public boolean execute(String sql){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


