package cn.ibizlab.core.ou.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.HashSet;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.ou.domain.IBZOrganization;
import cn.ibizlab.core.ou.filter.IBZOrganizationSearchContext;
import cn.ibizlab.core.ou.service.IIBZOrganizationService;

import cn.ibizlab.util.helper.CachedBeanCopier;



import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.ou.mapper.IBZOrganizationMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;


/**
 * 实体[单位机构] 服务对象接口实现
 */
@Slf4j
@Service("IBZOrganizationServiceImpl")
public class IBZOrganizationServiceImpl extends ServiceImpl<IBZOrganizationMapper, IBZOrganization> implements IIBZOrganizationService {

    @Autowired
    @Lazy
    private cn.ibizlab.core.ou.service.IIBZDepartmentService ibzdepartmentService;
    @Autowired
    @Lazy
    private cn.ibizlab.core.ou.service.IIBZEmployeeService ibzemployeeService;
    @Autowired
    @Lazy
    private cn.ibizlab.core.ou.service.IIBZOrganizationService ibzorganizationService;
    private int batchSize = 500;

    @Override
    public boolean checkKey(IBZOrganization et) {
        return (!ObjectUtils.isEmpty(et.getOrgid()))&&(!Objects.isNull(this.getById(et.getOrgid())));
    }

    @Override
    @Transactional
    public boolean update(IBZOrganization et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("orgid",et.getOrgid())))
            return false;
        CachedBeanCopier.copy(get(et.getOrgid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<IBZOrganization> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean create(IBZOrganization et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getOrgid()),et);
        return true;
    }

    @Override
    public void createBatch(List<IBZOrganization> list) {
        saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean save(IBZOrganization et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(IBZOrganization et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<IBZOrganization> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        if(!removeById(key)) return false;
        return true ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public IBZOrganization get(String key) {
        IBZOrganization et = getById(key);
        if(et==null){
            et=new IBZOrganization();
            et.setOrgid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public IBZOrganization getDraft(IBZOrganization et) {
        return et;
    }


	@Override
    public List<IBZOrganization> selectByParentorgid(String orgid) {
        return baseMapper.selectByParentorgid(orgid);
    }

    @Override
    public void removeByParentorgid(String orgid) {
        this.remove(new QueryWrapper<IBZOrganization>().eq("porgid",orgid));
    }


    /**
     * 查询集合 查询下级单位
     */
    @Override
    public Page<IBZOrganization> searchSelectSOrg(IBZOrganizationSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZOrganization> pages=baseMapper.searchSelectSOrg(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZOrganization>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 查询上级单位
     */
    @Override
    public Page<IBZOrganization> searchSelectPOrg(IBZOrganizationSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZOrganization> pages=baseMapper.searchSelectPOrg(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZOrganization>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<IBZOrganization> searchDefault(IBZOrganizationSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZOrganization> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZOrganization>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }


    @Override
    public List<JSONObject> select(String sql){
        return this.baseMapper.selectBySQL(sql);
    }

    @Override
    @Transactional
    public boolean execute(String sql){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


